@@ -2,6 +2,7 @@ import os
 import random
 import re
 import string
+import warnings
 from math import ceil
 from operator import itemgetter
 
@@ -38,6 +39,9 @@ from sklearn.tree import export_graphviz
 
 from datto.CleanDataframe import CleanDataframe
 
+# Hide precision/recall/f1 warnings for model scorings
+warnings.filterwarnings("always")
+
 
 class ModelResults:
     def most_similar_texts(
@@ -264,7 +268,13 @@ class ModelResults:
         )
 
     def coefficients_graph(
-        self, X_train, X_test, model, model_type, filename, path="../images/"
+        self,
+        X_train,
+        X_test,
+        model,
+        model_type,
+        filename="shap_graph",
+        path="../images/",
     ):
         """
         Displays graph of feature importances.
@@ -502,6 +512,8 @@ class ModelResults:
                 colnames=["Predicted"],
             )
             print(crosstab)
+            print("\n")
+
             sum_crosstab = crosstab.to_numpy().sum()
             print(
                 pd.crosstab(
